/*!
 * highlight-snail.js v0.0.0
 * https://github.com/Selection-Translator/highlight-snail
 * Released under the MIT License.
 */
import TinyEmitter from 'tiny-emitter';
import Popper from 'popper.js';

var supportsPassive = false;
try {
    window.addEventListener('test', null, {
        get passive() {
            supportsPassive = true;
            return false;
        }
    });
}
catch (e) { }
function addEvent(ele, event, handler, options) {
    var realOptions;
    if (supportsPassive) {
        if (typeof options === 'object') {
            realOptions = Object.assign({
                passive: true
            }, options);
        }
        else {
            realOptions = {
                passive: true,
                capture: options
            };
        }
    }
    else if (typeof options === 'object') {
        realOptions = options.capture;
    }
    ele.addEventListener(event, handler, realOptions);
    return function () {
        ele.removeEventListener(event, handler, realOptions);
    };
}
// tslint:disable-next-line:no-empty
function noop() { }
var selection = window.getSelection();
function getSelectionRange() {
    if (selection.type === 'Range')
        { return selection.getRangeAt(0); }
}
var rAF = window.requestAnimationFrame || function (cb) {
    window.setTimeout(cb, 1000 / 60);
};
function createStyleNode(str, doc) {
    if ( doc === void 0 ) doc = document;

    var node;
    // 根据字符串中是否包含 '{' 来判断这段字符串是 URL 还是 CSS 样式
    if (str.indexOf('{') > 0) {
        node = doc.createElement('style');
        node.textContent = str;
    }
    else {
        node = doc.createElement('link');
        node.rel = 'stylesheet';
        node.href = str;
    }
    return node;
}

var EMPTY_RECT = {
    x: 0,
    y: 0,
    left: 0,
    top: 0,
    right: 0,
    bottom: 0,
    width: 0,
    height: 0
};
var referenceObject = {
    getBoundingClientRect: function getBoundingClientRect() {
        var range = getSelectionRange();
        if (range) {
            return range.getBoundingClientRect();
        }
        else {
            return EMPTY_RECT;
        }
    },
    get clientHeight() {
        var range = getSelectionRange();
        if (range) {
            return range.getBoundingClientRect().height;
        }
        else {
            return 0;
        }
    },
    get clientWidth() {
        var range = getSelectionRange();
        if (range) {
            return range.getBoundingClientRect().width;
        }
        else {
            return 0;
        }
    }
};

var index = (function (TinyEmitter$$1) {
    function anonymous(el, options) {
        TinyEmitter$$1.call(this);
        this.el = el;
        this.options = options;
        this.unlisteners = [];
        this._stopScroll = noop;
        this.enableListeners();
    }

    if ( TinyEmitter$$1 ) anonymous.__proto__ = TinyEmitter$$1;
    anonymous.prototype = Object.create( TinyEmitter$$1 && TinyEmitter$$1.prototype );
    anonymous.prototype.constructor = anonymous;
    anonymous.prototype.enableListeners = function enableListeners () {
        var this$1 = this;

        this.unlisteners.push(addEvent(document, 'click', function () {
            this$1.hide();
        }), addEvent(document, 'mouseup', function () {
            window.setTimeout(function () {
                this$1.show();
            }, 0);
        }));
    };
    anonymous.prototype.disableListeners = function disableListeners () {
        this.unlisteners.forEach(function (unlisten) {
            unlisten();
        });
        this.unlisteners = [];
    };
    anonymous.prototype._init = function _init () {
        this._init = noop;
        var container = document.createElement('hs-container');
        var arrow = document.createElement('hs-arrow');
        var iFrame = document.createElement('iframe');
        container.appendChild(arrow);
        container.appendChild(iFrame);
        document.documentElement.appendChild(container);
        var contentDocument = iFrame.contentDocument;
        var head = contentDocument.head;
        head.appendChild(createStyleNode('body{margin:0;background-color:#fff}', contentDocument));
        contentDocument.body.appendChild(this.el);
        var ref = this;
        var options = ref.options;
        if (options) {
            // 在 iFrame 中插入样式
            var styles = options.styles;
            if (Array.isArray(styles)) {
                styles.forEach(function (str) {
                    head.appendChild(createStyleNode(str, contentDocument));
                });
            }
        }
        // 初始化弹窗
        var popper = new Popper(referenceObject, container, {
            placement: 'top',
            modifiers: {
                arrow: {
                    element: arrow
                }
            }
        });
        popper.disableEventListeners();
        this.container = container;
        this.iFrame = iFrame;
        this.popper = popper;
        this.emit('created');
    };
    // 在显示弹窗的期间监听全局 scroll 事件
    anonymous.prototype._startScroll = function _startScroll () {
        var this$1 = this;

        var running = false;
        var unwatch = addEvent(document, 'scroll', function () {
            if (running)
                { return; }
            rAF(function () {
                this$1.popper.update();
                running = false;
            });
        }, true);
        this._stopScroll = function () {
            unwatch();
            this._stopScroll = noop;
        };
    };
    anonymous.prototype.resize = function resize () {
        var ref = this;
        var el = ref.el;
        var ref$1 = this.iFrame;
        var style = ref$1.style;
        style.width = el.offsetWidth + 'px';
        style.height = el.offsetHeight + 'px';
    };
    anonymous.prototype.show = function show () {
        var range = getSelectionRange();
        if (range) {
            this._init();
            this.emit('beforeShow', range.toString());
            this.popper.update();
            // this.popper.enableEventListeners()
            this.resize();
            this.container.style.display = 'block';
            this._startScroll();
            return true;
        }
        else {
            return false;
        }
    };
    anonymous.prototype.hide = function hide () {
        var ref = this;
        var container = ref.container;
        if (container) {
            // this.popper.disableEventListeners()
            container.style.display = 'none';
            this._stopScroll();
        }
    };
    anonymous.prototype.destroy = function destroy () {
        this.disableListeners();
        this._stopScroll();
        this.off('created');
        this.off('beforeShow');
        var ref = this;
        var container = ref.container;
        if (container) {
            document.documentElement.removeChild(container);
        }
    };

    return anonymous;
}(TinyEmitter));

export default index;
