export interface IEventListenerArgs {
  capture?: boolean
  passive?: boolean
  once?: boolean
}

export type TEventListenerArgs = boolean | IEventListenerArgs

type TEventListener = (
  type: string,
  handler: null | ((event: Event) => void),
  options?: TEventListenerArgs
) => void

let supportsPassive = false
try {
  (window.addEventListener as TEventListener)('test', null, {
    get passive () {
      supportsPassive = true
      return false
    }
  })
} catch (e) {}

export function addEvent (
  ele: EventTarget,
  event: string,
  handler: (e: Event) => void,
  options?: TEventListenerArgs
) {
  let realOptions: TEventListenerArgs | undefined
  if (supportsPassive) {
    if (typeof options === 'object') {
      realOptions = Object.assign({
        passive: true
      }, options)
    } else {
      realOptions = {
        passive: true,
        capture: options
      }
    }
  } else if (typeof options === 'object') {
    realOptions = options.capture
  }

  ele.addEventListener(event, handler, realOptions)
  return function () {
    ele.removeEventListener(event, handler, realOptions)
  }
}

// tslint:disable-next-line:no-empty
export function noop () {}

const selection = window.getSelection()

export function getSelectionRange () {
  if (selection.type === 'Range') return selection.getRangeAt(0)
}

export const rAF = window.requestAnimationFrame || function (cb) {
  window.setTimeout(cb, 1000 / 60)
}

export function createStyleNode (str: string, doc = document) {
  let node
  // 根据字符串中是否包含 '{' 来判断这段字符串是 URL 还是 CSS 样式
  if (str.indexOf('{') > 0) {
    node = doc.createElement('style')
    node.textContent = str
  } else {
    node = doc.createElement('link')
    node.rel = 'stylesheet'
    node.href = str
  }
  return node
}
